from operator import sub, truediv # sub = <base for reduce>; truediv = <base for reduce>;
from functools import reduce # full compo of truediv;
from math import sqrt, prod # sqrt = Square Root (16 = 4), prod = *;
class new:
  def __init__(self):
    pass

  @staticmethod
  def add(f):
    def init(*args):
      return sum(args)
    return init

  @staticmethod
  def multi(f):
    def init(*args):
      return prod(args)
    return init

  @staticmethod
  def div(f):
    def init(*args):
      return reduce(truediv,args)
    return init

  @staticmethod
  def minus(f):
    def init(*args):
      return reduce(sub,args)
    return init

  @staticmethod
  def hyp(f):
    def init(opp,adj):
      return sqrt(opp ** 2 + adj ** 2)
    return init

  @staticmethod
  def opp(f):
    def init(hyp,adj):
      return sqrt(hyp ** 2 + adj ** 2)
    return init

  @staticmethod
  def adj(f):
    def init(hyp,opp):
      return sqrt(hyp ** 2 + opp ** 2)
    return init

  @staticmethod
  def adj(f):
    def init(hyp,opp):
      return sqrt(hyp ** 2 + opp ** 2)
    return init

  def nacci(f):
    def init(num_of_nums:int, index:int=None):
      nums = []
      num1, num2 = 0, 1
      for i in range(num_of_nums):
        num1, num2 = num2, num1 + num2
        nums.append(num1)
      if index == None:
        return nums
      else:
        return nums[index - 1]
    return init

def nacci(f):
  def init(num_of_nums:int, index:int=None):
    nums = []
    num1, num2 = 0, 1
    for i in range(num_of_nums):
      num1, num2 = num2, num1 + num2
      nums.append(num1)
    if index == None:
      return nums
    else:
      return nums[index - 1]
    return init

def add(f):
  def init(*args):
    return sum(args)
  return init

def multi(f):
  def init(*args):
    return prod(args)
  return init

def div(f):
  def init(*args):
    return reduce(truediv,args)
  return init

def minus(f):
  def init(*args):
    return reduce(sub,args)
  return init
  


def hyp(f):
  def init(opp,adj):
    return sqrt(opp ** 2 + adj ** 2)
  return init

def opp(f):
  def init(hyp,adj):
    return sqrt(hyp ** 2 + adj ** 2)
  return init

def adj(f):
  def init(hyp,opp):
    return sqrt(hyp ** 2 + opp ** 2)
  return init


import datetime
class new:
   def __init__(self,date:str):
     self.date = date
     self.date = self.date.replace("/","-")
     self.sdate = datetime.datetime.strptime(self.date,"%m-%d-%Y")
   def __str__(self):
     return self.date
   def text(self):
     return self.sdate.strftime("%b %d, %Y")


from ..base import Capylang

class init(Capylang):
  def __init__(self,string):
    if not type(string) is str:
      raise ValueError(f"Type \"{string.__class__.__name__}\" is not type \"str\"")
    self.string = string

  def freq(self):
    json = {}
    str = self.string.split()   
    str2 = []
 
    for i in str:            
        if i not in str2:
            str2.append(i)
             
    for i in range(0, len(str2)):
        json[str2[i]] = self.string.count(str2[i])
    
    return json

  def __repr__(self):
    return self.string

  def __str__(self):
    return self.string


import requests
global tpypi, pypi
try:
  pypi = requests.get("https://pypi.org/pypi/capylang/json").json()['info']['version']
  tpypi = requests.get("https://test.pypi.org/pypi/capylang/json").json()['info']['version']
except:
  raise Exception("Could not check version")
def check(__version__):
  if pypi != __version__:
    if __version__ < pypi:
      print(f"Capylang has a PyPI update! Version: {pypi}, update via capy.update()")
  elif tpypi != __version__:
    if __version__ < tpypi:
      print(f"Capylang has a TestPyPI update! Version: {tpypi}, update via capy.update()")


import os
def os_clear():
  os.system('cls' if os.name == 'nt' else 'clear')

def replit_clear():
  try:
    import replit
    replit.clear()
  except Exception as e:
    print(f"[WARNING] Maybe an ImportError. Trace stack: {e}")
  except BaseException as e:
    print(f"[WARNING] Maybe an ImportError. Trace stack: {e}")